(function() {
	var actualHtml,
      expectedHtml = '<h3>Hello Michael Sparks!</h3>';

	module('app');  
	test('app object should exist', function () {
    ok(app, 'app object exists!');
  });
  
  test('app.version should be 0.4.1', function () {
    strictEqual(app.version, '0.4.1', 'testing correct version!');
  });
  
  test('app.render method should exist', function () {
    ok(app.render, 'app.render method exists!');
  });
  
  module('app.render(view)');
  test('Test render where the template and data value property is not empty.', function () {
    //Setup
    var view = {  
        "name":"welcome-header",  
        "template":{value:'<h3>Hello {{#user}}{{firstName}} {{lastName}}{{/user}}!</h3>'},  
        "data":{value:{user:{firstName:'Michael',lastName:'Sparks'}}},  
        "container_id":"test-div"
      };
    app.render(view);
    actualHtml = document.getElementById('test-div').innerHTML;
    //Test
    strictEqual(actualHtml, expectedHtml, 'rendering view as object was successful!');
    //Tear down
    document.getElementById('test-div').innerHTML = '';
  });
  
  test('Test render where the template url property is not empty and data value property is not empty.', function () {
    //Setup
    var view = {  
        "name":"welcome-title",  
        "template":{url:"./resources/hello-title.view.template.html"},
        "data":{value:{user:{firstName:"Michael",lastName:"Sparks"}}},  
        "container_id":"test-div"
      };
    app.render(view);
    actualHtml = document.getElementById('test-div').innerHTML;
    //Test
    strictEqual(actualHtml, expectedHtml, 'rendering view as object was successful!');
    //Tear down
    document.getElementById('test-div').innerHTML = '';
  });
  
  test('Test render where the template url property is an app-path to a view with a template.', function () {
    //Setup
    app.home = {
      views:[{  
        "name":"welcome-template",  
        "template":{value:'<h3>Hello {{#user}}{{firstName}} {{lastName}}{{/user}}!</h3>'},  
        "data":{value:{user:{firstName:'Michael',lastName:'Sparks'}}},  
        "container_id":"test-div"
      }]
    };
    var view = {  
        "name":"welcome-title",  
        "template":{url:"app-path://home/welcome-template"},
        "data":{value:{user:{firstName:"Michael",lastName:"Sparks"}}},  
        "container_id":"test-div"
      };
    app.render(view);
    actualHtml = document.getElementById('test-div').innerHTML;
    //Test
    strictEqual(actualHtml, expectedHtml, 'rendering view as object was successful!');
    //Tear down
    document.getElementById('test-div').innerHTML = '';
    delete app.home;
  });
  
  test('Test render where the template value property is not empty and data url property is not empty.', function () {
    //Setup
    var view = {  
        "name":"welcome-title",  
        "template":{value:'<h3>Hello {{#user}}{{firstName}} {{lastName}}{{/user}}!</h3>'},  
        "data":{url:"./resources/hello-title.view.data.json"}, 
        "container_id":"test-div"
      };
    ok(!view.data.value, 'The view\'s data value should not be present');
    app.render(view);
    actualHtml = document.getElementById('test-div').innerHTML;
    //Test
    strictEqual(actualHtml, expectedHtml, 'rendering view as object was successful!');
    deepEqual(view.data.value, {user:{firstName:'Michael',lastName:'Sparks'}}, 'The view\'s data value should be equal to that which was fetched from the url.');
    //Tear down
    document.getElementById('test-div').innerHTML = '';
  });
  
  test('Test render where both template and data url properties are not empty.', function () {
    //Setup
    var view = {  
        "name":"hello-title",  
        "template":{url:"./resources/hello-title.view.template.html"},
        "data":{url:"./resources/hello-title.view.data.json"}, //I would normally use a .json file type but my webserver is not currently serving that type.
        "container_id":"test-div"
      };
    ok(!view.template.value, 'The view\'s template value should not be present');
    ok(!view.data.value, 'The view\'s data value should not be present');
    app.render(view);
    actualHtml = document.getElementById('test-div').innerHTML;
    //Test
    strictEqual(actualHtml, expectedHtml, 'rendering view as object was successful!');
    //Tear down
    document.getElementById('test-div').innerHTML = '';
  });
  
  module('app.render(appPath)');
  test('Test render where the view name in the appPath is a number.  The number is used as an index to access the views array', function () {
    //Setup
    app.home = {
      views:[{  
        "name":"welcome-header",  
        "template":{value:'<h3>Hello {{#user}}{{firstName}} {{lastName}}{{/user}}!</h3>'},  
        "data":{value:{user:{firstName:'Michael',lastName:'Sparks'}}},  
        "container_id":"test-div"
      }]
    };
    app.render('app-path://home/0');
    actualHtml = document.getElementById('test-div').innerHTML;
    //Test
    strictEqual(actualHtml, expectedHtml, 'rendering view as object was successful!');
    //Tear down
    document.getElementById('test-div').innerHTML = '';
    delete app.home;
  });
  
  test('Test render where the view name in the appPath is a string.  The string, i.e. view name, is used to find the first view in the array with a matching name.', function () {
    //Setup
    app.welcome = {
      views:[{  
        "name":"title",  
        "template":{value:'<h3>Hello {{#user}}{{firstName}} {{lastName}}{{/user}}!</h3>'},  
        "data":{value:{user:{firstName:'Michael',lastName:'Sparks'}}},  
        "container_id":"test-div"
      }]
    };
    app.render('app-path://welcome/title');
    actualHtml = document.getElementById('test-div').innerHTML;
    //Test
    strictEqual(actualHtml, expectedHtml, 'rendering view as object was successful!');
    //Tear down
    document.getElementById('test-div').innerHTML = '';
    delete app.welcome;
  });
  
  module('app.render(url)');
  test('Test render where view is fetched from a url.', function () {
    //Test
    app.render('./resources/hello-title.view.json');
    //Tear down
    document.getElementById('test-div').innerHTML = '';
  });
}());
